/***************************************************************************
                          fireapplet.cpp  -  description
                             -------------------
    begin                : Thu Jul 19 12:39:42 EST 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kaboutapplication.h>
#include <qlayout.h>
#include <stdio.h>
#include <stdlib.h>
#include <qfile.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qslider.h>
#include <qpushbutton.h>

#include "fireapplet.h"
#include "fireapplet.moc"

extern "C"
{
  KPanelApplet* init( QWidget *parent, const QString configFile)
  {
    KGlobal::locale()->insertCatalogue("fireapplet");
    return new FireApplet(configFile, KPanelApplet::Normal,
                      KPanelApplet::About | KPanelApplet::Preferences,
                      parent, "fireapplet");
  }
}

FireApplet::FireApplet(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
       : KPanelApplet(configFile, type, actions, parent, name)
{
	// Get the current application configuration handle
  ksConfig = config();

	aboutData = 0;
	prefDialog = 0;

	readConfig();

	fire = new FireWidget(this);
	QHBoxLayout *layout= new QHBoxLayout(this);
  layout->add(fire);

	animationTimer = new QTimer(this);
	QObject::connect(animationTimer, SIGNAL(timeout()), fire, SLOT(renderNextFrame()));
	animationTimer->start(20);

	updateTimer = new QTimer(this);
	QObject::connect(updateTimer, SIGNAL(timeout()), this, SLOT(updateCPULoad()));

	setBehaviour();	
}

FireApplet::~FireApplet()
{
	if(prefDialog)
		delete prefDialog;
}

void FireApplet::about()
{
  if(!aboutData)
	{
		aboutData = new KAboutData("fireapplet", I18N_NOOP("Fire Applet"),
              	"0.1", I18N_NOOP("This KDE panel applet draws a flame showing on the current CPU usage."),
              	KAboutData::License_GPL, "(c) 2002, Thorsten Scheuermann");

		aboutData->addAuthor("Thorsten Scheuermann", 0, "tscheuermann@gmx.net");
  }

  KAboutApplication dialog(aboutData);

  dialog.exec();
}

void FireApplet::preferences()
{
	if(!prefDialog)
	{
		prefDialog = new PreferencesDialog();

		// connect ok and apply buttons
		QObject::connect(prefDialog->okButton, SIGNAL(clicked()), this, SLOT(apply()));
		QObject::connect(prefDialog->applyButton, SIGNAL(clicked()), this, SLOT(apply()));
  }

	// update dialog settings
	prefDialog->radioButton1->setChecked(!showCPUUsage);
	prefDialog->radioButton2->setChecked(showCPUUsage);
	prefDialog->userCheckBox->setChecked(showUserLoad);
	prefDialog->systemCheckBox->setChecked(showSystemLoad);
	prefDialog->niceCheckBox->setChecked(showNiceLoad);
	prefDialog->intensitySlider->setValue(fixedFlameIntensity);

	prefDialog->show();
}

int FireApplet::widthForHeight( int height ) const
{
   return height;
}

int FireApplet::heightForWidth( int width ) const
{
   return width;
}

void FireApplet::updateCPULoad()
{
	static unsigned long oldUser = 0;
	static unsigned long oldSystem = 0;
	static unsigned long oldNice = 0;
	static unsigned long oldIdle = 0;
	unsigned long currentUser;
	unsigned long currentSystem;
	unsigned long currentNice;
	unsigned long currentIdle;
	unsigned long totalTicks;
  int load;

  QFile statFile("/proc/stat");
	char lineBuf[256];

	statFile.open(IO_ReadOnly);
	statFile.readLine(lineBuf, 256);
	statFile.close();

	sscanf(lineBuf, "%*s %lu %lu %lu %lu", &currentUser, &currentSystem,
				 &currentNice, &currentIdle);


	totalTicks = (currentUser-oldUser) + (currentSystem - oldSystem) +
							 (currentNice-oldNice) + (currentIdle - oldIdle);

	if(totalTicks > 10)
	{
		load = 0;
		if(showUserLoad)
			load += currentUser-oldUser;
		if(showSystemLoad)
			load += currentSystem-oldSystem;
		if(showNiceLoad)
			load += currentNice-oldNice;

		load = 100*load/totalTicks;
	}
	else
		load = 0;

	oldUser = currentUser;
	oldSystem = currentSystem;
	oldNice = currentNice;
	oldIdle = currentIdle;

	emit cpuLoad(load);
}

void FireApplet::apply()
{
	if(!prefDialog)
		return;

	showCPUUsage = prefDialog->radioButton2->isChecked();
	showUserLoad = prefDialog->userCheckBox->isChecked();
	showSystemLoad = prefDialog->systemCheckBox->isChecked();
	showNiceLoad = prefDialog->niceCheckBox->isChecked();
	fixedFlameIntensity = prefDialog->intensitySlider->value();

	setBehaviour();

	writeConfig();
}

void FireApplet::setBehaviour()
{
	if(showCPUUsage)
	{
		QObject::connect(this, SIGNAL(cpuLoad(int)), fire, SLOT(setFireIntensity(int)));
		updateTimer->start(333);
  }
	else
	{
		updateTimer->stop();
		QObject::disconnect(this, SIGNAL(cpuLoad(int)), fire, SLOT(setFireIntensity(int)));
		fire->setFireIntensity(fixedFlameIntensity);
	}
}

void FireApplet::writeConfig()
{
	ksConfig->setGroup("Parameters");
	ksConfig->writeEntry("showCPUUsage", showCPUUsage);
	ksConfig->writeEntry("showUserLoad", showUserLoad);
	ksConfig->writeEntry("showSystemLoad", showSystemLoad);
	ksConfig->writeEntry("showNiceLoad", showNiceLoad);
	ksConfig->writeEntry("fixedFlameIntensity", fixedFlameIntensity);
	ksConfig->sync();
}

void FireApplet::readConfig()
{
	ksConfig->setGroup("Parameters");
	showCPUUsage = ksConfig->readBoolEntry("showCPUUsage", true);
	showUserLoad = ksConfig->readBoolEntry("showUserLoad", true);
	showSystemLoad = ksConfig->readBoolEntry("showSystemLoad", true);
	showNiceLoad = ksConfig->readBoolEntry("showNiceLoad", true);
	fixedFlameIntensity = ksConfig->readNumEntry("fixedFlameIntensity", 50);
}
